#include "Quad2d.h"

//	Useful #defines

#define GL_TRI_X(s, t) (0.25 * (1 + s) * (1 + t))		
#define GL_TRI_Y(s, t) (0.25 * (1 + s) * (1 - t))
#define GL_TRI_W(s, wa, wb) (0.125 * (1 + s) * wa * wb)

#define GL_QUAD_X(s, t) (s)		
#define GL_QUAD_Y(s, t) (t)
#define GL_QUAD_W(s, wa, wb) (1.0)

// --- tensor production functions ----------------


// these take a 1d quadrature rule and produce the tensor equivalent.

QuadRule MakeTri2DRule(const QuadRule &rule1D)
{
	Int			i, j, elts = rule1D.Rows();
	QuadRule	result;
	
	result.SetSize(sqr(elts), 3);
	
	for (i = 0; i < elts; i++)
		for (j = 0; j < elts; j++)
		{
			result[i * elts + j][0] = GL_TRI_W(rule1D[i][1], rule1D[i][0], rule1D[j][0]);
			result[i * elts + j][1] = GL_TRI_X(rule1D[i][1], rule1D[j][1]);
			result[i * elts + j][2] = GL_TRI_Y(rule1D[i][1], rule1D[j][1]);
		}
		
	return(result);
}

QuadRule MakeQuad2DRule(const QuadRule &rule1D)
{
	Int			i, j, elts = rule1D.Rows();
	QuadRule	result;

	result.SetSize(sqr(elts), 3);
	
	for (i = 0; i < elts; i++)
		for (j = 0; j < elts; j++)
		{
			result[i * elts + j][0] = rule1D[i][0] * rule1D[j][0];
			result[i * elts + j][1] = rule1D[j][1];
			result[i * elts + j][2] = rule1D[i][1];
		}	
	return(result);
}

// --- 1-point GL Quadrature over a triangle & quad --------------------------------------------

const QuadRule GL_TRI_1 = MakeTri2DRule(GL_1);
const QuadRule GL_QUAD_1 = MakeQuad2DRule(GL_1);

// --- 2-point x 2-point GL Quadrature over a triangle & quad --------------------------------------------

const QuadRule GL_TRI_2 = MakeTri2DRule(GL_2);
const QuadRule GL_QUAD_2 = MakeQuad2DRule(GL_2);

// --- 3-point x 3-point GL Quadrature over a triangle & quad --------------------------------------------

const QuadRule GL_TRI_3 = MakeTri2DRule(GL_3);
const QuadRule GL_QUAD_3 = MakeQuad2DRule(GL_3);

// --- 4-point x 4-point GL Quadrature over a triangle & quad --------------------------------------------

const QuadRule GL_TRI_4 = MakeTri2DRule(GL_4);
const QuadRule GL_QUAD_4 = MakeQuad2DRule(GL_4);
const QuadRule GL_TRI_5 = MakeTri2DRule(GL_5);
const QuadRule GL_QUAD_5 = MakeQuad2DRule(GL_5);
const QuadRule GL_TRI_6 = MakeTri2DRule(GL_6);
const QuadRule GL_QUAD_6 = MakeQuad2DRule(GL_6);

// --- 2-point x 2-point GJ Quadrature over a triangle & quad --------------------------------------------

const QuadRule GJ_TRI_2 = MakeTri2DRule(GJ_2);
const QuadRule GJ_QUAD_2 = MakeQuad2DRule(GJ_2);

// --- 3-point x 3-point GJ Quadrature over a triangle & quad --------------------------------------------

const QuadRule GJ_TRI_3 = MakeTri2DRule(GJ_3);
const QuadRule GJ_QUAD_3 = MakeQuad2DRule(GJ_3);

	
